@@ -4,6 +4,7 @@
 import os
 import code
 from twisted.internet import threads
+from twisted.internet import defer
 import subprocess
 import email
 import urwid
@@ -17,6 +18,7 @@ from alot.commands import Command, registerCommand
 from alot.completion import CommandLineCompleter
 from alot.commands import CommandParseError
 from alot.commands import commandfactory
+from alot.commands import CommandCanceled
 from alot import buffers
 from alot.widgets.utils import DialogBox
 from alot import helper
@@ -28,6 +30,7 @@ from alot.db.envelope import Envelope
 from alot import commands
 from alot.settings import settings
 from alot.helper import split_commandstring, split_commandline
+from alot.helper import mailto_to_envelope
 from alot.utils.booleanaction import BooleanAction
 
 MODE = 'global'
@@ -123,6 +126,8 @@ class PromptCommand(Command):
             # save into prompt history
             ui.commandprompthistory.append(cmdline)
             ui.apply_commandline(cmdline)
+        else:
+            raise CommandCanceled()
 
 
 @registerCommand(MODE, 'refresh')
@@ -402,7 +407,7 @@ class BufferCloseCommand(Command):
                                                     select='yes', cancel='no',
                                                     msg_position='left')) ==
                     'no'):
-                return
+                raise CommandCanceled()
 
         if len(ui.buffers) == 1:
             if settings.get('quit_on_last_bclose'):
@@ -479,20 +484,23 @@ class OpenBufferlistCommand(Command):
             ui.buffer_open(bl)
 
 
-@registerCommand(MODE, 'taglist')
+@registerCommand(MODE, 'taglist', arguments=[
+    (['--tags'], {'nargs': '+', 'help': 'tags to display'}),
+])
 class TagListCommand(Command):
 
     """opens taglist buffer"""
-    def __init__(self, filtfun=None, **kwargs):
+    def __init__(self, filtfun=None, tags=None, **kwargs):
         """
         :param filtfun: filter to apply to displayed list
         :type filtfun: callable (str->bool)
         """
         self.filtfun = filtfun
+        self.tags = tags
         Command.__init__(self, **kwargs)
 
     def apply(self, ui):
-        tags = ui.dbman.get_all_tags()
+        tags = self.tags or ui.dbman.get_all_tags()
         blists = ui.get_buffers_of_type(buffers.TagListBuffer)
         if blists:
             buf = blists[0]
@@ -569,12 +577,7 @@ class HelpCommand(Command):
             title_att = settings.get_theming_attribute('help', 'title')
             section_att = settings.get_theming_attribute('help', 'section')
             # get mappings
-            if ui.mode in settings._bindings:
-                modemaps = dict(settings._bindings[ui.mode].items())
-            else:
-                modemaps = {}
-            is_scalar = lambda k_v: k_v[0] in settings._bindings.scalars
-            globalmaps = dict(filter(is_scalar, settings._bindings.items()))
+            globalmaps, modemaps = settings.get_keybindings(ui.mode)
 
             # build table
             maxkeylength = len(max((modemaps).keys() + globalmaps.keys(),
@@ -636,18 +639,19 @@ class HelpCommand(Command):
                             'help': 'do not add signature'}),
     (['--spawn'], {'action': BooleanAction, 'default': None,
                    'help': 'spawn editor in new terminal'}),
+    (['rest'], {'nargs': '*'}),
 ])
 class ComposeCommand(Command):
 
     """compose a new email"""
     def __init__(self, envelope=None, headers={}, template=None,
                  sender=u'', subject=u'', to=[], cc=[], bcc=[], attach=None,
-                 omit_signature=False, spawn=None, **kwargs):
+                 omit_signature=False, spawn=None, rest=[], **kwargs):
         """
         :param envelope: use existing envelope
         :type envelope: :class:`~alot.db.envelope.Envelope`
         :param headers: forced header values
-        :type header: doct (str->str)
+        :type headers: dict (str->str)
         :param template: name of template to parse into the envelope after
                          creation. This should be the name of a file in your
                          template_dir
@@ -668,6 +672,10 @@ class ComposeCommand(Command):
         :type omit_signature: bool
         :param spawn: force spawning of editor in a new terminal
         :type spawn: bool
+        :param rest: remaining parameters. These can start with
+                     'mailto' in which case it is interpreted sa mailto string.
+                     Otherwise it will be interpreted as recipients (to) header
+        :type rest: list(str)
         """
 
         Command.__init__(self, **kwargs)
@@ -683,11 +691,19 @@ class ComposeCommand(Command):
         self.attach = attach
         self.omit_signature = omit_signature
         self.force_spawn = spawn
+        self.rest = ' '.join(rest)
 
     @inlineCallbacks
     def apply(self, ui):
         if self.envelope is None:
-            self.envelope = Envelope()
+            if self.rest:
+                if self.rest.startswith('mailto'):
+                    self.envelope = mailto_to_envelope(self.rest)
+                else:
+                    self.envelope = Envelope()
+                    self.envelope.add('To', self.rest)
+            else:
+                self.envelope = Envelope()
         if self.template is not None:
             # get location of tempsdir, containing msg templates
             tempdir = settings.get('template_dir')
@@ -743,8 +759,8 @@ class ComposeCommand(Command):
                 fromaddress = yield ui.prompt('From', completer=cmpl,
                                               tab=1)
                 if fromaddress is None:
-                    ui.notify('canceled')
-                    return
+                    raise CommandCanceled()
+
                 self.envelope.add('From', fromaddress)
 
         # add signature
@@ -799,8 +815,8 @@ class ComposeCommand(Command):
             to = yield ui.prompt('To',
                                  completer=completer)
             if to is None:
-                ui.notify('canceled')
-                return
+                raise CommandCanceled()
+
             self.envelope.add('To', to.strip(' \t\n,'))
 
         if settings.get('ask_subject') and \
@@ -808,8 +824,8 @@ class ComposeCommand(Command):
             subject = yield ui.prompt('Subject')
             logging.debug('SUBJECT: "%s"' % subject)
             if subject is None:
-                ui.notify('canceled')
-                return
+                raise CommandCanceled()
+
             self.envelope.add('Subject', subject)
 
         if settings.get('compose_ask_tags'):
@@ -817,8 +833,8 @@ class ComposeCommand(Command):
             tagsstring = yield ui.prompt('Tags', completer=comp)
             tags = filter(lambda x: x, tagsstring.split(','))
             if tags is None:
-                ui.notify('canceled')
-                return
+                raise CommandCanceled()
+
             self.envelope.tags = tags
 
         if self.attach:
@@ -857,34 +873,11 @@ class MoveCommand(Command):
         elif self.movement == 'first':
             if hasattr(ui.current_buffer, "focus_first"):
                 ui.current_buffer.focus_first()
+                ui.update()
         elif self.movement == 'last':
             if hasattr(ui.current_buffer, "focus_last"):
                 ui.current_buffer.focus_last()
+                ui.update()
         else:
             ui.notify('unknown movement: ' + self.movement,
                       priority='error')
-
-
-class CommandSequenceCommand(Command):
-
-    """Meta-Command that just applies a sequence of given Commands in order"""
-
-    def __init__(self, cmdline='', **kwargs):
-        Command.__init__(self, **kwargs)
-        self.cmdline = cmdline
-
-    @inlineCallbacks
-    def apply(self, ui):
-        # split commandline if necessary
-        for cmdstring in split_commandline(self.cmdline):
-            logging.debug('CMDSEQ: apply %s' % str(cmdstring))
-            # translate cmdstring into :class:`Command`
-            try:
-                cmd = commandfactory(cmdstring, ui.mode)
-                # store cmdline for use with 'repeat' command
-                if cmd.repeatable:
-                    ui.last_commandline = self.cmdline.lstrip()
-            except CommandParseError as e:
-                ui.notify(e.message, priority='error')
-                return
-            yield ui.apply_command(cmd)
