@@ -9,6 +9,7 @@ import argparse
 from twisted.internet.defer import inlineCallbacks
 import subprocess
 from email.Utils import getaddresses, parseaddr
+from email.message import Message
 import mailcap
 from cStringIO import StringIO
 
@@ -17,6 +18,7 @@ from alot.commands.globals import ExternalCommand
 from alot.commands.globals import FlushCommand
 from alot.commands.globals import ComposeCommand
 from alot.commands.globals import MoveCommand
+from alot.commands.globals import CommandCanceled
 from alot.commands.envelope import SendCommand
 from alot import completion
 from alot.db.utils import decode_header
@@ -29,6 +31,7 @@ from alot.db.errors import DatabaseROError
 from alot.settings import settings
 from alot.helper import parse_mailcap_nametemplate
 from alot.helper import split_commandstring
+from alot.helper import email_as_string
 from alot.utils.booleanaction import BooleanAction
 from alot.completion import ContactsCompleter
 
@@ -56,9 +59,13 @@ def determine_sender(mail, action='reply'):
     assert my_accounts, 'no accounts set!'
 
     # extract list of addresses to check for my address
+    # X-Envelope-To and Envelope-To are used to store the recipient address
+    # if not included in other fields
     candidate_addresses = getaddresses(mail.get_all('To', []) +
                                        mail.get_all('Cc', []) +
                                        mail.get_all('Delivered-To', []) +
+                                       mail.get_all('X-Envelope-To', []) +
+                                       mail.get_all('Envelope-To', []) +
                                        mail.get_all('From', []))
 
     logging.debug('candidate addresses: %s' % candidate_addresses)
@@ -149,7 +156,7 @@ class ReplyCommand(Command):
             subject = reply_subject_hook(subject)
         else:
             rsp = settings.get('reply_subject_prefix')
-            if not subject.startswith(('Re:', rsp)):
+            if not subject.lower().startswith(('re:', rsp.lower())):
                 subject = rsp + subject
         envelope.add('Subject', subject)
 
@@ -300,11 +307,16 @@ class ForwardCommand(Command):
 
             envelope.body = mailcontent
 
+            for a in self.message.get_attachments():
+                envelope.attach(a)
+
         else:  # attach original mode
             # attach original msg
-            mail.set_type('message/rfc822')
-            mail['Content-Disposition'] = 'attachment'
-            envelope.attach(Attachment(mail))
+            original_mail = Message()
+            original_mail.set_type('message/rfc822')
+            original_mail['Content-Disposition'] = 'attachment'
+            original_mail.set_payload(email_as_string(mail))
+            envelope.attach(Attachment(original_mail))
 
         # copy subject
         subject = decode_header(mail.get('Subject', ''))
@@ -385,8 +397,8 @@ class BounceMailCommand(Command):
             completer = None
         to = yield ui.prompt('To', completer=completer)
         if to is None:
-            ui.notify('canceled')
-            return
+            raise CommandCanceled()
+
         mail['Resent-To'] = to.strip(' \t\n,')
 
         logging.debug("bouncing mail")
@@ -823,7 +835,7 @@ class SaveAttachmentCommand(Command):
                     ui.notify('not a directory: %s' % self.path,
                               priority='error')
             else:
-                ui.notify('canceled')
+                raise CommandCanceled()
         else:  # save focussed attachment
             focus = ui.get_deep_focus()
             if isinstance(focus, AttachmentWidget):
@@ -842,7 +854,7 @@ class SaveAttachmentCommand(Command):
                     except (IOError, OSError) as e:
                         ui.notify(str(e), priority='error')
                 else:
-                    ui.notify('canceled')
+                    raise CommandCanceled()
 
 
 class OpenAttachmentCommand(Command):
