@@ -51,9 +51,9 @@ def add_signature_headers(mail, sigs, error_msg):
     )
     mail.add_header(
         X_SIGNATURE_MESSAGE_HEADER,
-        'Invalid: {0}'.format(error_msg)
+        u'Invalid: {0}'.format(error_msg)
         if error_msg else
-        'Valid: {0}'.format(sig_from),
+        u'Valid: {0}'.format(sig_from),
     )
 
 
@@ -104,19 +104,19 @@ def message_from_file(handle):
 
         malformed = False
         if len(m.get_payload()) != 2:
-            malformed = 'expected exactly two messages, got {0}'.format(
+            malformed = u'expected exactly two messages, got {0}'.format(
                 len(m.get_payload()))
 
         ct = m.get_payload(1).get_content_type()
         if ct != app_pgp_sig:
-            malformed = 'expected Content-Type: {0}, got: {1}'.format(
+            malformed = u'expected Content-Type: {0}, got: {1}'.format(
                 app_pgp_sig, ct)
 
         # TODO: RFC 3156 says the alg has to be lower case, but I've
         # seen a message with 'PGP-'. maybe we should be more
         # permissive here, or maybe not, this is crypto stuff...
         if not p.get('micalg', 'nothing').startswith('pgp-'):
-            malformed = 'expected micalg=pgp-..., got: {0}'.format(
+            malformed = u'expected micalg=pgp-..., got: {0}'.format(
                 p.get('micalg', 'nothing'))
 
         sigs = []
@@ -125,7 +125,7 @@ def message_from_file(handle):
                 sigs = crypto.verify_detached(m.get_payload(0).as_string(),
                                               m.get_payload(1).get_payload())
             except GPGProblem as e:
-                malformed = str(e)
+                malformed = unicode(e)
 
         add_signature_headers(m, sigs, malformed)
 
@@ -144,13 +144,14 @@ def message_from_file(handle):
 
         ct = m.get_payload(0).get_content_type()
         if ct != app_pgp_enc:
-            malformed = 'expected Content-Type: {0}, got: {1}'.format(
+            malformed = u'expected Content-Type: {0}, got: {1}'.format(
                 app_pgp_enc, ct)
 
         want = 'application/octet-stream'
         ct = m.get_payload(1).get_content_type()
         if ct != want:
-            malformed = 'expected Content-Type: {0}, got: {1}'.format(want, ct)
+            malformed = u'expected Content-Type: {0}, got: {1}'.format(want,
+                                                                       ct)
 
         if not malformed:
             try:
@@ -160,7 +161,7 @@ def message_from_file(handle):
                 # the combined method is used, currently this prevents
                 # the interpretation of the recovered plain text
                 # mail. maybe that's a feature.
-                malformed = str(e)
+                malformed = unicode(e)
             else:
                 # parse decrypted message
                 n = message_from_string(d)
@@ -199,8 +200,10 @@ def message_from_file(handle):
                     add_signature_headers(m, sigs, '')
 
         if malformed:
-            msg = 'Malformed OpenPGP message: {0}'.format(malformed)
-            m.attach(email.message_from_string(msg))
+            msg = u'Malformed OpenPGP message: {0}'.format(malformed)
+            content = email.message_from_string(msg.encode('utf-8'))
+            content.set_charset('utf-8')
+            m.attach(content)
 
     return m
 
@@ -334,7 +337,7 @@ def decode_header(header, normalize=False):
     decode a header value to a unicode string
 
     values are usually a mixture of different substrings
-    encoded in quoted printable using diffetrent encodings.
+    encoded in quoted printable using different encodings.
     This turns it into a single unicode string
 
     :param header: the header value
@@ -396,11 +399,12 @@ def encode_header(key, value):
         value = Header(value)
     return value
 
+
 def is_subdir_of(subpath, superpath):
-    #make both absolute    
+    # make both absolute
     superpath = os.path.realpath(superpath)
     subpath = os.path.realpath(subpath)
 
-    #return true, if the common prefix of both is equal to directory
-    #e.g. /a/b/c/d.rst and directory is /a/b, the common prefix is /a/b
+    # return true, if the common prefix of both is equal to directory
+    # e.g. /a/b/c/d.rst and directory is /a/b, the common prefix is /a/b
     return os.path.commonprefix([subpath, superpath]) == superpath
