@@ -11,16 +11,18 @@ import email
 import mimetypes
 import os
 import re
+from email.generator import Generator
 from email.mime.audio import MIMEAudio
 from email.mime.base import MIMEBase
 from email.mime.image import MIMEImage
 from email.mime.text import MIMEText
+from email.mime.multipart import MIMEMultipart
 import urwid
 import magic
 from twisted.internet import reactor
 from twisted.internet.protocol import ProcessProtocol
 from twisted.internet.defer import Deferred
-import StringIO
+from cStringIO import StringIO
 import logging
 
 
@@ -81,14 +83,13 @@ def string_sanitize(string, tab_width=8):
     :type tab_width: int or `None`
 
     >>> string_sanitize(' foo\rbar ', 8)
-    'foobar'
+    ' foobar '
     >>> string_sanitize('foo\tbar', 8)
     'foo     bar'
     >>> string_sanitize('foo\t\tbar', 8)
     'foo             bar'
     """
 
-    string = string.strip()
     string = string.replace('\r', '')
 
     lines = list()
@@ -338,8 +339,8 @@ def call_cmd_async(cmdlist, stdin=None, env=None):
     class _EverythingGetter(ProcessProtocol):
         def __init__(self, deferred):
             self.deferred = deferred
-            self.outBuf = StringIO.StringIO()
-            self.errBuf = StringIO.StringIO()
+            self.outBuf = StringIO()
+            self.errBuf = StringIO()
             self.outReceived = self.outBuf.write
             self.errReceived = self.errBuf.write
 
@@ -465,7 +466,7 @@ def mimewrap(path, filename=None, ctype=None):
     content = open(path, 'rb').read()
     if not ctype:
         ctype = guess_mimetype(content)
-        # libmagic < 5.12 incorrectly detects excel/powerpoint files as 
+        # libmagic < 5.12 incorrectly detects excel/powerpoint files as
         # 'application/msword' (see #179 and #186 in libmagic bugtracker)
         # This is a workaround, based on file extension, useful as long
         # as distributions still ship libmagic 5.11.
@@ -551,3 +552,94 @@ def parse_mailcap_nametemplate(tmplate='%s'):
     else:
         template_suffix = tmplate
     return (template_prefix, template_suffix)
+
+
+def parse_mailto(mailto_str):
+    """
+    Interpret mailto-string
+
+    :param mailto_str: the string to interpret. Must conform to :rfc:2368.
+    :return: pair headers,body. headers is a dict mapping str to lists of
+        (str, body) is a str.
+    :rtype: (dict(str-->[str,..], str)
+    """
+    if mailto_str.startswith('mailto:'):
+        import urllib
+        to_str, parms_str = mailto_str[7:].partition('?')[::2]
+        headers = {}
+        body = u''
+
+        to = urllib.unquote(to_str)
+        if to:
+            headers['To'] = [to]
+
+        for s in parms_str.split('&'):
+            key, value = s.partition('=')[::2]
+            key = key.capitalize()
+            if key is 'body':
+                body = urllib.unquote(value)
+            elif value:
+                headers[key] = [urllib.unquote(value)]
+        return (headers, body)
+    else:
+        return (None, None)
+
+
+def mailto_to_envelope(mailto_str):
+    """
+    Interpret mailto-string into a :class:`alot.db.envelope.Envelope`
+    """
+    from alot.db.envelope import Envelope
+    headers, body = parse_mailto(mailto_str)
+    return Envelope(bodytext=body, headers=headers)
+
+
+def RFC3156_canonicalize(text):
+    """
+    Canonicalizes plain text (MIME-encoded usually) according to RFC3156.
+
+    This function works as follows (in that order):
+
+    1. Convert all line endings to \\\\r\\\\n (DOS line endings).
+    2. Ensure the text ends with a newline (\\\\r\\\\n).
+    3. Encode all occurences of "From " at the beginning of a line
+       to "From=20" in order to prevent other mail programs to replace
+       this with "> From" (to avoid MBox conflicts) and thus invalidate
+       the signature.
+
+    :param text: text to canonicalize (already encoded as quoted-printable)
+    :rtype: str
+    """
+    text = re.sub("\r?\n", "\r\n", text)
+    if not text.endswith("\r\n"):
+        text += "\r\n"
+    text = re.sub("^From ", "From=20", text, flags=re.MULTILINE)
+    return text
+
+
+def email_as_string(mail):
+    """
+    Converts the given message to a string, without mangling "From" lines
+    (like as_string() does).
+
+    :param mail: email to convert to string
+    :rtype: str
+    """
+    fp = StringIO()
+    g = Generator(fp, mangle_from_=False, maxheaderlen=78)
+    g.flatten(mail)
+    as_string = RFC3156_canonicalize(fp.getvalue())
+
+    if isinstance(mail, MIMEMultipart):
+        # Get the boundary for later
+        boundary = mail.get_boundary()
+
+        # Workaround for http://bugs.python.org/issue14983:
+        # Insert a newline before the outer mail boundary so that other mail
+        # clients can verify the signature when sending an email which contains
+        # attachments.
+        as_string = re.sub(r'--(\r\n)--' + boundary,
+                           '--\g<1>\g<1>--' + boundary,
+                           as_string, flags=re.MULTILINE)
+
+    return as_string
